unit XUtil;

interface

uses
  Windows, Messages, SysUtils, Classes, WinTypes, WinProcs, Graphics, Controls,
  Forms, Dialogs, Buttons, ExtCtrls, ImageHlp, IniFiles, StdCtrls, Grids;

var
	XAppName: string;				// Application exe name without path and extension
	XAppDir: string;				// Application directory, path to exe file
	XWorkDir: string;				// Working directory (current directory)
	XTempDir: string;				// Windows temporary directory
	XComputerName: string;	// Computer name
	XUserName: string;			// User name

	SYS_CheckBoxWidth: integer;
	SYS_CheckBoxHeight: integer;

var
	XWorkIni: TIniFile;

(*
	{Get|Set}Component{Str|Int} makes interface to ini file XWorkDir+XAppName+'.ini'
	so that component (form, datamodule, control) specific values can be saved and loaded in
	the program when needed.

	This is used by XControls to save states... and can be used by program also  
*)
function GetComponentStr(c: TComponent; Item, Default: string) : string;
procedure SetComponentStr(c: TComponent; Item, Value: string);
function GetComponentInt(c: TComponent; Item: string; Default: longint) : longint;
procedure SetComponentInt(c: TComponent; Item: string; Value: longint);

function Sign(value : double) : double;
function Replace(Text: string; SearchStr, ReplaceStr: string) : string;
function Seconds : Longint;
function ReadMemo(Path: string) : string;
procedure WriteMemo(Path, Text: string);
function StrWinToDos(s: string): string;
function StrDosToWin(s: string): string;
function StrTok(src, brk: string) : string;

function RoundX (x: double; d: integer): double;
function TruncX (x: double; d: integer): double;
function RoundCUR (x: double): currency;
function DivideX (x,y: double): double;

procedure XWinError (ErrMsg : string);
procedure XFreeObject (var obj);
function XGetActiveControlText: string;

procedure FixDPI(f: TForm; DesignDPI: integer);

const
	JOCKER_MANY = '%';
	JOCKER_ONE = '?';

function XStrMatch(cStr, cMatch: string): boolean;

function XDateMask(IncludeLiterals: boolean): string;
function XTimeMask(IncludeLiterals: boolean): string;

(*
		XScanDateTime function is used for decoding Date and/or Time from string
		that was required since delphi code don't work as it should when formats are like yyyymmdd & hhnnss
		format is according to the specification for FormatDateTime function but with limitations.
		Limitations:
			- Only decimal representation of date and time is supported. Parsing of abbreviations,
				short/long date/time names is not supported.
			-	Supported format specifiers are: yy, yyyy, m, mm, d, dd, h, hh, n, nn, s, ss, /, :, 'xx'/"xx"
				others like c, t, tt, am/pm, a/p, ampm and ddd... & mmm... are not supported
*)

function XScanDateTime(const str, fmt: string; var str_pos: integer): TDateTime;

function XStrToDateTime(const str, fmt: string): TDateTime;

procedure XListDLLFunctions(DLLName: string; List: TStrings);

(*
	XCodeStr kodira (i dekodira prethodno kodirani) string preko kodne rijei.
	Ako se kodna rije ne zada, onda je default kodiranje!
*)
function XCodeStr(s: string; code: string = ''): string;

function XSpecialFolder(nFolder: Integer): string;

function MessageDlgEx(const Msg: string; DlgType: TMsgDlgType; Buttons: TMsgDlgButtons; DefaultButton: TMsgDlgBtn = mbOk): integer;

implementation

Uses UTrace, ShlObj, ActiveX;

// *******************************************************
//	Component Values
// *******************************************************

var
	CompVal_Section, CompVal_Entry: string;

procedure Comp_SectionEntry(c: TComponent; Item: string);

	function IsTopLevel(x: TComponent): boolean;
	begin
		result := (x is TCustomForm) or (x is TDataModule) or (x is TApplication) or (x.Owner = nil);
	end;

var
	o: TComponent;
begin
	if IsTopLevel(c) then
		begin
			// Component je top level, idemo direktno section i entry
			CompVal_Section := copy(c.ClassName, 2, maxint);
			CompVal_Entry := Item;
		end
	else
		begin
			// Component nije top level, traimo joj owner component koja je top level
			o := c.Owner;
			CompVal_Entry := c.Name + '.' + Item;
			while not IsTopLevel(o) do
				begin
					CompVal_Entry := o.Name + '.' + CompVal_Entry;
					o := o.Owner;
				end;
			CompVal_Section := copy(o.ClassName, 2, maxint);
		end;
end;

function GetComponentStr(c: TComponent; Item, Default: string) : string;
begin
	Comp_SectionEntry(c, Item);
	Result := XWorkIni.ReadString(CompVal_Section, CompVal_Entry, Default);
end;

procedure SetComponentStr(c: TComponent; Item, Value: string);
begin
	Comp_SectionEntry(c, Item);
	if Value <> '' then
		XWorkIni.WriteString(CompVal_Section, CompVal_Entry, Value)
	else
		XWorkIni.DeleteKey(CompVal_Section, CompVal_Entry);
end;

function GetComponentInt(c: TComponent; Item: string; Default: longint) : longint;
begin
	Comp_SectionEntry(c, Item);
	Result := XWorkIni.ReadInteger(CompVal_Section, CompVal_Entry, Default);
end;

procedure SetComponentInt(c: TComponent; Item: string; Value: longint);
begin
	Comp_SectionEntry(c, Item);
	XWorkIni.WriteInteger(CompVal_Section, CompVal_Entry, Value);
end;

//******************//
// Utility funkcije //
//******************//

function Sign(value : double) : double;
begin
	if (value < 0.0) then
  	Result := -1.0
  else if (value > 0.0) then
		Result := 1.0
  else
  	Result := 0.0;
end;

function Replace(Text: string; SearchStr, ReplaceStr: string) : string;
var
  NewStr, OldStr: string;
	p: integer;
begin
	NewStr := '';
  OldStr := Text;

  while Length(OldStr) > 0 do
  	begin
			p := Pos(SearchStr, OldStr);
      if p <> 0 then
      	begin
      		NewStr := NewStr + Copy(OldStr, 1, p-1) + ReplaceStr;
          OldStr := Copy(OldStr, p+Length(SearchStr), Length(OldStr));
        end
      else
      	begin
      		NewStr := NewStr + OldStr;
          OldStr := '';
        end;
    end;

	Result := NewStr;
end;

function Seconds : Longint;
var
 HH, MM, SS, MS : Word;
begin
 DecodeTime(Time,HH,MM,SS,MS);
 Result:=(HH*3600)+(MM*60)+SS;
end;

function ReadMemo(Path: string) : string;
const
	FILE_BUFFER = 4096;
Var
  fh: integer;
  size, read: integer;
begin
	Result := '';
	fh := FileOpen(Path, fmOpenRead);
  if fh <= 0 then
  	raise Exception.Create('ReadMemo: Ne mogu otvoriti datoteku ' + Path + #10 +
			'Error - ' + IntToStr(GetLastError()) + ' (' + SysErrorMessage(GetLastError())+')')
  else
  	try
		  size := GetFileSize(fh, nil);
      SetLength(Result, size);
   	  read := FileRead(fh, Result[1], size);
      SetLength(Result, read);
		finally
      FileClose(fh);
    end;
end;

procedure WriteMemo(Path, Text: string);
Var
  fh: integer;
begin
	fh := FileCreate(Path);
  if fh <= 0 then
  	raise Exception.Create('WriteMemo: Ne mogu kreirati datoteku ' + Path + #10 +
			'Error - ' + IntToStr(GetLastError()) + ' (' + SysErrorMessage(GetLastError())+')')
  else
  	try
   	  if FileWrite(fh, Text[1], Length(Text)) <> Length(Text) then
      	raise Exception.Create('WriteMemo: Ne mogu pisati u datoteku ' + Path);
    finally  
      FileClose(fh);
    end;
end;

function StrWinToDos(s: string): string;
const
	WIN_CH: string = #154+#240+#232+#230+#158+#138+#208+#200+#198+#142;
	DOS_CH: string = #231+#208+#159+#134+#167+#230+#209+#172+#143+#166;
var
	i: integer;
  p: byte;
begin
	Result := s;
	for i := 1 to Length(Result) do
  	begin
    	p := Pos(Result[i], WIN_CH);
	  	if p > 0 then
      	Result[i] := DOS_CH[p];
    end;
end;

function StrDosToWin(s: string): string;
const
	WIN_CH: string = #154+#240+#232+#230+#158+#138+#208+#200+#198+#142;
	DOS_CH: string = #231+#208+#159+#134+#167+#230+#209+#172+#143+#166;
var
	i: integer;
  p: byte;
begin
	Result := s;
	for i := 1 to Length(Result) do
  	begin
    	p := Pos(Result[i], DOS_CH);
	  	if p > 0 then
      	Result[i] := WIN_CH[p];
    end;
end;

var
	StrTokStatic: string;

function StrTok(src, brk: string) : string;
var
	i, j: integer;
begin
	Result := '';

	if (src <> '') then
  	StrTokStatic := src;

	if StrTokStatic <> '' then
  	begin
      // Preskacemo znakove sa pocetka
      i := 1;
      while (i <= Length(StrTokStatic)) and (Pos(StrTokStatic[i], brk) <> 0) do
      	Inc(i);
      // Ako je na kraju niza
      if i > Length(StrTokStatic) then
      	begin
					StrTokStatic := '';
        end
      else
      	begin
          // Nasli smo pocetak niza i trazimo kraj
          j := i+1;
          while (j <= Length(StrTokStatic)) and (Pos(StrTokStatic[j], brk) = 0) do
            Inc(j);
          Result := Copy(StrTokStatic, i, j-i);
          StrTokStatic := Copy(StrTokStatic, j+1, Length(StrTokStatic));
				end;
    end;
end;

const
	Base: array [0..9] of double = (
	1.0,
    10.0,
    100.0,
    1000.0,
    10000.0,
    100000.0,
    1000000.0,
    10000000.0,
    100000000.0,
    1000000000.0
  );

function RoundX (x: double; d: integer): double;
begin
	if x > 0 then
	 	Result := Int(x*Base[d] + 0.500001)/Base[d]
  else
	 	Result := Int(x*Base[d] - 0.500001)/Base[d];
end;

function TruncX (x: double; d: integer): double;
begin
	Result := Int(x*Base[d])/Base[d];
end;

function RoundCUR (x: double): currency;
begin
	if x > 0 then
	 	x := Int(x*100 + 0.5001)
	else
	 	x := Int(x*100 - 0.5001);
 	Result := x / 100.0;
end;

function DivideX (x,y: double): double;
begin
  if y = 0 then
    Result := 0
  else
    Result := x/y;
end;

procedure XWinError (ErrMsg: string);
begin
	raise Exception.Create (ErrMsg + #10 +
		'Error - ' + IntToStr(GetLastError()) + ' (' + SysErrorMessage(GetLastError()) + ')');
end;

procedure XFreeObject (var obj);
begin
	if TObject(obj) <> nil then
		begin
			if (TObject(obj) is TCustomForm) and TCustomForm(obj).Showing then
				TCustomForm(obj).Close;
			TObject(obj).Free;
			TObject(obj) := nil;
		end;
end;

function XGetActiveControlText: string;
var
	h: HWND;
  Len: integer;
begin
	h := 0;
	if (Screen.ActiveControl is TCustomGrid) then
		h := GetTopWindow(Screen.ActiveControl.Handle);
	if (h = 0) or (not IsWindowVisible(h)) or (not IsWindowEnabled(h)) then
		h := Screen.ActiveControl.Handle;

	Len := GetWindowTextLength(h);
	SetLength(Result, Len+1);
	if Len > 0 then
		GetWindowText(h, PChar(Result), Len+1);
	SetLength(Result, Len);

	//DebugInfo('ActiveControl: <' + Screen.ActiveControl.Name + '/' + Screen.ActiveControl.ClassName + '>');
	//DebugInfo('GetGetActiveControlText: <' + Result + '>');
end;

procedure FixDPI(f: TForm; DesignDPI: integer);

(*
	Small/Large system font solution
	Have a TForm's properties Scaled and AutoScroll set to false
	and in OnCreate event of your form call this procedure like:
		FixDPI (TForm1, 96) // If you designed in small fonts

	Author: Tomislav Kardas
*)

var
	ScaleM, ScaleD: integer;

	function Scale(x: longint): longint;
	begin
		result := (x*ScaleM + ScaleD div 2) div ScaleD;
	end;

	procedure FixControl(c: TControl);
	var
		x: integer;
		p: TWinControl;
		fixheight: boolean;
	begin
		p := c.Parent;
		if c.Align <> alNone then
			begin
				// Fixing width
				if ((c.Align = alLeft) and (akRight in c.Anchors)) or
					 ((c.Align = alRight) and (akLeft in c.Anchors)) then
					c.Width := p.ClientWidth - Scale(p.ClientWidth - c.Width)
				else if (c.Align = alLeft) or (c.Align = alRight) then
					c.Width := Scale(c.Width);
				// Fixing height
				if ((c.Align = alTop) and (akBottom in c.Anchors)) or
					 ((c.Align = alBottom) and (akTop in c.Anchors)) then
					c.Height := p.ClientHeight - Scale(p.ClientHeight - c.Height)
				else if (c.Align = alTop) or (c.Align = alBottom) then
					c.Height := Scale(c.Height);
			end
		else
			begin
				// Fixing width
				x := p.ClientWidth - c.Width - c.Left;
				if akLeft in c.Anchors then
					begin
						c.Left := Scale(c.Left);
						if akRight in c.Anchors then
							c.Width := p.ClientWidth - c.Left - Scale(x)
						else
							c.Width := Scale(c.Width);
					end
				else if akRight in c.Anchors then
					begin
						c.Width := Scale(c.Width);
						c.Left := p.ClientWidth - c.Width - Scale(x);
					end
				else
					begin
						c.Left := Scale(c.Left);
						c.Width := Scale(c.Width);
					end;

				// Fixing height
				fixheight := true;
				if (c is TCustomEdit) and not (c is TCustomMemo) then
					fixheight := false;

				x := p.ClientHeight - c.Height - c.Top;
				if akTop in c.Anchors then
					begin
						c.Top := Scale(c.Top);
						if fixheight then
							begin
								if akBottom in c.Anchors then
									c.Height := p.ClientHeight - c.Top - Scale(x)
								else
									c.Height := Scale(c.Height);
							end;
					end
				else if akBottom in c.Anchors then
					begin
						if fixheight then
							c.Height := Scale(c.Height);
						c.Top := p.ClientHeight - c.Height - Scale(x);
					end
				else
					begin
						c.Top := Scale(c.Top);
						if fixheight then
							c.Height := Scale(c.Height);
					end;
			end;
	end;

	procedure FixControls(c: TWinControl);
	var
		i: integer;
	begin
		for i := 0 to c.ControlCount - 1 do
			if c.Controls[i].Owner = f then
				begin
					FixControl(c.Controls[i]);
					if c.Controls[i] is TWinControl then
						FixControls(TWinControl(c.Controls[i]));
				end;
	end;

begin
	if DesignDPI <> Screen.PixelsPerInch then
		begin
			ScaleM := Screen.PixelsPerInch;
			ScaleD := DesignDPI;
			f.ClientWidth := Scale(f.ClientWidth);
			f.ClientHeight := Scale(f.ClientHeight);
			FixControls(f);
		end;
end;

function XStrMatch(cStr, cMatch: string): boolean;
var
	nStrLen, nMatchLen: byte;
	s_idx, m_idx, n: byte;
	m_ch: char;
begin
	nStrLen := Length(cStr);
	nMatchLen := Length(cMatch);

	s_idx := 1;
	m_idx := 1;

	while (nStrLen > 0) and (nMatchLen > 0) do
		begin
			m_ch := cMatch[m_idx];
			Inc(m_idx);
			Dec(nMatchLen);

			if m_ch = JOCKER_ONE then
				begin
					// ? zamjenjuje bilo koji znak
					Inc(s_idx);
					Dec(nStrLen);
				end
			else if m_ch = JOCKER_MANY then
				begin
					// * zamjenjuje bilo koju sekvencu
					n := 0;
					while (n < nMatchLen) and
								(cMatch[m_idx+n] <> JOCKER_ONE) and
								(cMatch[m_idx+n] <> JOCKER_MANY) do
						Inc(n);
					// n predstavlja duljinu dijela u match stringu
					// koji se nalazi iza * pa do prvog znaka
					// ?,* ili kraja niza
					while (nStrLen >= n) and (nMatchLen > 0) and
								(Copy(cStr, s_idx, n) <> Copy(cMatch, m_idx, n)) do
						begin
							Inc(s_idx);
							Dec(nStrLen);
						end;
				end
			else if cStr[s_idx] <> m_ch then
				begin
					// Uzorci nisu jednaki
					Result := False;
					Exit;
				end
			else
				begin
					// Uzorci su jednaki pa preskace taj znak u stringu
					Inc(s_idx);
					Dec(nStrLen);
				end;

		end;

	// Ako je cijeli match izasao onda se match poklapa sa stringom
	Result := nMatchLen = 0;
end;

function XDateMask(IncludeLiterals: boolean): string;
	var
  	SysFormat: string;
    EntryFormat: string;
    i: integer;
    ch: char;
	begin
    //DateSeparator: '.'
    //ShortDateFormat: 'dd.mm.yy'
    //Result: '!99/99/99;0;_' ili '!99/99/99;1;_'

    (*
  	SysFormat := UpperCase(ShortDateFormat);
		for i := 1 to Length(SysFormat) do
    	begin
      	if (SysFormat[i] = 'D') or (SysFormat[i] = 'M') then
         	SysFormat[i] := '9'
        else if (SysFormat[i] = 'Y') then
         	SysFormat[i] := '9'
        else if (SysFormat[i] = DateSeparator) then
         	SysFormat[i] := '/';
      end;
    *)

  	SysFormat := UpperCase(ShortDateFormat);
    EntryFormat := '';
    i := Length(SysFormat);
		while i > 0  do
    	begin
        ch := SysFormat[i];
      	if (ch = 'D') or (ch = 'M') then
          begin
            EntryFormat := '99' + EntryFormat;
            while (i > 1) and (SysFormat[i-1] = ch) do
              Dec(i);
          end
        else if (ch = 'Y') then
          EntryFormat :=  '9' + EntryFormat
        else if (ch = DateSeparator) then
          EntryFormat := '/' + EntryFormat;
        Dec(i);
      end;

    if IncludeLiterals then
			Result := '!' + EntryFormat + ';1;_'
    else
			Result := '!' + EntryFormat + ';0;_';
  end;

function XTimeMask(IncludeLiterals: boolean): string;
var
  SysFormat: string;
  i: integer;
begin
  SysFormat := UpperCase(LongTimeFormat);
  for i := 1 to Length(SysFormat) do
    begin
      if (SysFormat[i] = 'H') or (SysFormat[i] = 'M') then
        SysFormat[i] := '9'
      else if (SysFormat[i] = 'S') then
        SysFormat[i] := '9'
      else if (SysFormat[i] = TimeSeparator) then
        SysFormat[i] := ':';
    end;

  if IncludeLiterals then
    Result := '!' + SysFormat + ';1;_'
  else
    Result := '!' + SysFormat + ';0;_';
end;

function XScanDateTime(const str, fmt: string; var str_pos: integer): TDateTime;

function UpCaseChar(c: char): char;
begin
	result := c;
	if (c >= 'a') and (c <= 'z') then
		Dec(result, 32);
end;

function CurrentYear: Word;
var
	SystemTime: TSystemTime;
begin
	GetLocalTime(SystemTime);
	Result := SystemTime.wYear;
end;

type
	TDTItem = record
		value: word;
		digits: byte;
		parsed: boolean;
	end;

procedure AddDigit(var item: TDTItem; d: char);
begin
	item.value := (item.value*10) + Ord(d) - Ord('0');
	Inc(item.digits);
	item.parsed := true;
end;

const
	YEAR = 0;
	MONTH = 1;
	DAY = 2;
	HOUR = 3;
	MIN = 4;
	SEC = 5;
	FmtSpecs: PChar = 'YMDHNS';
var
	Items: array [0..5] of TDTItem;
	i: byte;						// current item
	fmt_pos: byte;			// format (fmt) string position
	quote: byte;				// 0 - out of quotes, 1 - in single quotes, 2 - in double quotes
	sc, fc: char;				// current string & format characters
	pFspec: PChar;			// pointer to format specifier in FmtSpecs
	parse: boolean;			// parse flag until done
	CenturyBase: word;	// temporary variable for two digit year fix to four digit
begin
	// Zero memory init
	FillChar(Items, sizeof(Items), 0);

	// Skip space;
	while (str_pos <= Length(str)) and (str[str_pos] = ' ') do
		Inc(str_pos);

	// starting from first char of format as out of quotes
	fmt_pos := 1; quote := 0; parse := true;
	while parse and (str_pos <= Length(str)) and (fmt_pos <= Length(fmt)) do
		begin
			// get current format char
			fc := UpCaseChar(fmt[fmt_pos]);
			Inc(fmt_pos);

			if (fc = '''') and (quote in [0, 1]) then
				begin
					quote := 1 - quote;
					fc := #0;
				end
			else if (fc = '"') and (quote in [0, 2]) then
				begin
					quote := 2 - quote;
					fc := #0;
				end
			else if (fc = '/') and (quote = 0) then
				begin
					fc := DateSeparator;
				end
			else if (fc = ':') and (quote = 0) then
				begin
					fc := TimeSeparator;
				end
			else if (fc = 'M') and (Items[HOUR].digits > 0) then
				begin
					// According to FormatDateTime format specification,
					// m & mm format after the h & hh is treated as minutes and not as month
					fc := 'N';
				end;

			if fc <> #0 then
				begin
					// get current string char
					sc := UpCaseChar(str[str_pos]);

					pFspec := StrScan(FmtSpecs, fc);
					if pFspec <> nil then
						begin
							i := pFspec - FmtSpecs;
							if sc in ['0'..'9'] then
								begin
									AddDigit(Items[i], sc);

									// if only first digit is parsed and next format char is not the same
									// (we can not use fc in cmp here because M is changed into N below!) 
									if (Items[i].digits = 1) and (UpCaseChar(fmt[fmt_pos-1]) <> UpCaseChar(fmt[fmt_pos])) and (str[str_pos+1] in ['0'..'9']) then
										begin
											// then format specifier contains single format char,
											// and we have to allow one more optional digit into the same format char
											AddDigit(Items[i], str[str_pos+1]);
											Inc(str_pos);
										end;
								end
							else
								parse := false;
						end
					else if sc <> fc then
						parse := false;

					if parse then
						Inc(str_pos);
				end;
		end;

	// if parse is false then format was not fully parsed from input string,
	// so we have to check and report error if:
	// 1. nothing is entered
	// 2. date or time is entered incomplete
	if not parse then
		if (
				((Items[MONTH].digits = 0) or (Items[DAY].digits = 0)) and
				((Items[HOUR].digits = 0) or (Items[MIN].digits = 0))
		) or (
				(Items[MONTH].parsed <> Items[DAY].parsed) or
				(Items[HOUR].parsed <> Items[MIN].parsed)
		)	then
			raise Exception.CreateFmt('''%s'' is not a valid date and/or time (format: %s)', [str, fmt]);

	if (Items[YEAR].value = 0) and (Items[MONTH].value = 0) and (Items[DAY].value = 0) then
		result := 0
	else
		begin
			if Items[YEAR].digits = 0 then
				Items[YEAR].value := CurrentYear
			else if Items[YEAR].value < 100 then
				begin
					CenturyBase := CurrentYear - TwoDigitYearCenturyWindow;
					Inc(Items[YEAR].value, CenturyBase div 100 * 100);
					if (TwoDigitYearCenturyWindow > 0) and (Items[YEAR].value < CenturyBase) then
						Inc(Items[YEAR].value, 100);
				end;
			result := EncodeDate(Items[YEAR].value, Items[MONTH].value, Items[DAY].value);
		end;

	if (Items[HOUR].value <> 0) or (Items[MIN].value <> 0) or (Items[SEC].value <> 0) then
		result := result + EncodeTime(Items[HOUR].value, Items[MIN].value, Items[SEC].value, 0);
end;

function XStrToDateTime(const str, fmt: string): TDateTime;
var
  i: integer;
begin
  i := 1;
  result := XScanDateTime(str, fmt, i);
end;

procedure XListDLLFunctions(DLLName: string; List: TStrings);
// by Dmitry Streblechenko
(*
	This one is not reliable since it depends on debug information, which
	most DLLs will not have.
*)
type
	chararr = array[0..$FFFFFF] of char;
var
	h: THandle;
	i,fc: integer;
	st: string;
	arr: pointer;
	ImageDebugInformation: PImageDebugInformation;
begin
	List.Clear;
	DLLName := ExpandFileName(DLLName);
	if FileExists(DLLName) then
  	begin
			h := CreateFile(PChar(DLLName), GENERIC_READ,
									FILE_SHARE_READ or FILE_SHARE_WRITE, nil, OPEN_EXISTING,
                 	FILE_ATTRIBUTE_NORMAL, 0);
			if h <> INVALID_HANDLE_VALUE then
      	try
					ImageDebugInformation := MapDebugInformation(h, PChar(DLLName), nil,0);
					if ImageDebugInformation <> nil then
          	try
              arr := ImageDebugInformation^.ExportedNames;
              fc := 0;
              for i := 0 to ImageDebugInformation^.ExportedNamesSize-1 do
                begin
                  if chararr(arr^)[i] = #0 then
                    begin
                     st := PChar(@chararr(arr^)[fc]);
                     if length(st)>0 then List.Add(st);
                     if (i>0) and (chararr(arr^)[i-1]=#0) then Break;
                     fc := i+1;
                    end;
                end;
            finally
							UnmapDebugInformation(ImageDebugInformation);
            end;
        finally
					CloseHandle(h);
        end;
    end;
end;

function XCodeStr(s: string; code: string): string;
var
	n, i, c_mod, c_len: integer;
begin
	n := Length(s);
	c_len := Length(code);
	SetLength(result, n);
	if c_len = 1 then
		c_mod := Ord(code[1]) and $1f
	else
		c_mod := 0;
	for i := 1 to n do
		begin
			// Preskaemo kodiranje kontrolnih znakova
			if Ord(s[i]) <= $1f then
				result[i] := s[i]
			else
				begin
					if c_len > 1 then
						c_mod := ord(code[((i-1) mod c_len)+1]) and $1f;
					result[i] := chr(Ord(s[i]) xor (i and $1f) xor c_mod);
				end;
		end;
end;

function XSpecialFolder(nFolder: Integer): string;
var
  Item: PItemIDList;
  Path: Array[0..MAX_PATH - 1] of char;
begin
	SHGetSpecialFolderLocation(0, nFolder, Item);
	SHGetPathFromIDList(Item, @Path[0]);
	CoTaskMemFree(Item);
	Result := Path;
end;

function MessageDlgEx(const Msg: string; DlgType: TMsgDlgType; Buttons: TMsgDlgButtons; DefaultButton: TMsgDlgBtn): integer;
const
	ButtonNames: array[TMsgDlgBtn] of string = (
		'Yes', 'No', 'OK', 'Cancel', 'Abort', 'Retry', 'Ignore', 'All', 'NoToAll', 'YesToAll', 'Help'
	);
var
	f: TForm;
	i: integer;
	c: TButton;
begin
	f := CreateMessageDialog(Msg, DlgType, Buttons);
	try
		f.Position := poScreenCenter;
		if DefaultButton <> mbOk then
			for i := 0 to f.ControlCount-1 do
				begin
					if f.Controls[i] is TButton then
						begin
							c := TButton(f.Controls[i]);
							c.Default := (c.Name = ButtonNames[DefaultButton]);
							if c.Default then
								f.ActiveControl := c;
							//DebugInfoFmt('Button: %s, DefaultButton=%s, IsDefault=%d', [c.Name, ButtonNames[DefaultButton], ord(c.Default)]);
						end;
				end;

		Result := f.ShowModal;
	finally
		f.Free;
	end;
end;

procedure GlobalInit;
const
	PathSize: integer = 255;
	MAX_NAME_LENGTH = 255;
var
	Buffer: PChar;
	i: integer;
	NameLength: DWORD;
  ini_file: file of Byte;
  ini_size: longint;
begin
	// Ime aplikacije
	XAppName := ExtractFileName(ParamStr(0));
	if UpperCase(copy(XAppName, Length(XAppName)-3, 4)) = '.EXE' then
		XAppName := copy(XAppName, 1, Length(XAppName)-4);

	// Default putanje
	XAppDir := UpperCase(ExtractFilePath(ParamStr(0)));
	XWorkDir := UpperCase(GetCurrentDir+'\');

	for i := 1 to ParamCount do begin
		if UpperCase(Copy(ParamStr(i), 1, 3)) = '/A=' then
			XAppDir := Copy(ParamStr(i), 4, Length(ParamStr(i)))
	end;

	Buffer := StrAlloc(PathSize);
	try
		// Postavljanje putanje na tmp direktorij
		GetTempPath(PathSize, Buffer);
		XTempDir := Buffer;
	finally
		StrDispose(Buffer);
	end;

	// Konverzija dosadanjeg imena AppIni datoteke
	if FileExists(XWorkDir + 'Pmx.ini') then
		RenameFile(XWorkDir + 'Pmx.ini', XWorkDir + XAppName + '.ini');

	if FileExists(XWorkDir + XAppName + '.ini') then
		begin
			// Ako je ini file vei od 62 KB onda ga briemo jer inae
			// javlja greku kod rada sa njime. Alternativa bi bila
			// da se koristi TMemIniFile.
			AssignFile(ini_file, XWorkDir + XAppName + '.ini');
			Reset(ini_file);
			ini_size := FileSize(ini_file);
			CloseFile(ini_file);
			if ini_size >= (62*1024) then
				DeleteFile(PChar(XWorkDir + XAppName + '.ini'));
		end;
	XWorkIni := TIniFile.Create(XWorkDir + XAppName + '.ini');

	NameLength := MAX_NAME_LENGTH;
	SetLength(XComputerName, NameLength);
	if not GetComputerName(PChar(XComputerName), NameLength) then
		XComputerName := '<NoComputerName>'
	else
		SetLength(XComputerName, NameLength);

	NameLength := MAX_NAME_LENGTH;
	SetLength(XUserName, NameLength);
	if not GetUserName(PChar(XUserName), NameLength) then
		XUserName := '<NoUserName>'
	else
		SetLength(XUserName, NameLength-1);

	DebugInfo('XAppDir = ' + XAppDir);
	DebugInfo('XWorkDir = ' + XWorkDir);
	DebugInfo('XTempDir = ' + XTempDir);
end;

initialization

	randomize;

	TwoDigitYearCenturyWindow := 50;

	GlobalInit;

	with TBitmap.Create do
		try
			Handle := LoadBitmap(0, MAKEINTRESOURCE(OBM_CHECKBOXES));
			SYS_CheckBoxWidth := Width div 4;
			SYS_CheckBoxHeight := Height div 3;
		finally
			Free;
		end;

finalization

	XWorkIni.Free;

end.

