unit XMLTestMain;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	ComCtrls, StdCtrls, XMLNode, ExtCtrls, MSXML2_TLB, Menus, ActnList,
  Buttons;

type
	TfmXMLTestMain = class(TForm)
    tvNodes: TTreeView;
    opdXMLFile: TOpenDialog;
		svdXMLFile: TSaveDialog;
    sbStatus: TStatusBar;
    paTool: TPanel;
    buOpen: TButton;
    buNew: TButton;
    btnFullExpand: TButton;
    buSave: TButton;
    buFree: TButton;
    btnWriteTest: TButton;
    laTimeToOpen: TLabel;
    laTimeToSave: TLabel;
    laTimeToFree: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    buMSXML: TButton;
    XMLDoc: TXMLDoc;
		pbProgress: TProgressBar;
    ActionList: TActionList;
    MainMenu: TMainMenu;
		acOpen: TAction;
    acNew: TAction;
    acSave: TAction;
    acFree: TAction;
    acMSXML: TAction;
    acWriteTest: TAction;
    File1: TMenuItem;
    Open1: TMenuItem;
    New1: TMenuItem;
    Save1: TMenuItem;
    Free1: TMenuItem;
    N1: TMenuItem;
    WriteTest1: TMenuItem;
    MSXML1: TMenuItem;
    acIncTree: TAction;
    Options1: TMenuItem;
    IncrementalTreeFill1: TMenuItem;
    XMLDocumentreadoptions1: TMenuItem;
    WriteOptions1: TMenuItem;
    acFormatOutput: TAction;
    acMixedContent: TAction;
    acPreserveWhiteSpace: TAction;
    PreserveWhiteSpace1: TMenuItem;
    MixedContent1: TMenuItem;
    FormatOutput1: TMenuItem;
    Label4: TLabel;
    meNode: TRichEdit;
		spNode: TSplitter;
    sbNode: TSpeedButton;
    btnSelect: TButton;
		procedure btnFullExpandClick(Sender: TObject);
		procedure FormCreate(Sender: TObject);
		procedure FormDestroy(Sender: TObject);
		procedure opdXMLFileShow(Sender: TObject);
		procedure tvNodesExpanding(Sender: TObject; Node: TTreeNode;
			var AllowExpansion: Boolean);
    procedure sbStatusResize(Sender: TObject);
    procedure XMLDocReadProgress(Sender: TObject; Reader: TXMLReader);
    procedure XMLDocWriteProgress(Sender: TObject; Writer: TXMLWriter);
    procedure tvNodesChange(Sender: TObject; Node: TTreeNode);
    procedure acOpenUpdate(Sender: TObject);
    procedure acOpenExecute(Sender: TObject);
    procedure acNewUpdate(Sender: TObject);
    procedure acNewExecute(Sender: TObject);
    procedure acSaveUpdate(Sender: TObject);
    procedure acSaveExecute(Sender: TObject);
    procedure acFreeUpdate(Sender: TObject);
    procedure acFreeExecute(Sender: TObject);
    procedure acMSXMLUpdate(Sender: TObject);
    procedure acMSXMLExecute(Sender: TObject);
    procedure acWriteTestUpdate(Sender: TObject);
    procedure acWriteTestExecute(Sender: TObject);
    procedure acIncTreeUpdate(Sender: TObject);
		procedure acIncTreeExecute(Sender: TObject);
    procedure acPreserveWhiteSpaceUpdate(Sender: TObject);
    procedure acPreserveWhiteSpaceExecute(Sender: TObject);
    procedure acMixedContentUpdate(Sender: TObject);
		procedure acMixedContentExecute(Sender: TObject);
		procedure acFormatOutputUpdate(Sender: TObject);
		procedure acFormatOutputExecute(Sender: TObject);
		procedure sbNodeClick(Sender: TObject);
		procedure btnSelectClick(Sender: TObject);
    procedure tvNodesCustomDrawItem(Sender: TCustomTreeView;
      Node: TTreeNode; State: TCustomDrawState; var DefaultDraw: Boolean);
	private
		{ Private declarations }
		FIncTree: boolean;
		FBeginMemSize: integer;
		FBeginTime: Int64;
		FSelectedNodes: TXMLNodeArray;
		procedure UpdateTreeView;
		procedure TimeStart;
		procedure TimeStop(l: TLabel);
		procedure SetStatus(s: string);
		procedure SetNode(Node: TXMLNode);
	public
		{ Public declarations }
	end;

var
	fmXMLTestMain: TfmXMLTestMain;

implementation

{$R *.DFM}

uses XUtil, UTrace, RichEdit;

procedure TfmXMLTestMain.FormCreate(Sender: TObject);
begin
	FIncTree := true;
	pbProgress.Parent := sbStatus;
end;

procedure TfmXMLTestMain.FormDestroy(Sender: TObject);
begin
	//
end;

procedure TfmXMLTestMain.UpdateTreeView;

	procedure AddNode(Node: TXMLNode; TreeNode: TTreeNode);
	var
		x: TTreeNode;
		n: TXMLNode;
	begin
		x := tvNodes.Items.AddChild(TreeNode, Node.Description);
		(*
		DebugInfo(x.Text + '.'#13#10 +
			'  Text = [' + StringReplace(Node.Text, #10, #13#10, [rfReplaceAll]) + ']'#13#10 +
			'  XML = ' + Node.XML + #13#10);
		*)

		if Node.ChildNodesCount > 0 then
			begin
				n := Node.FirstChild;
				while n <> nil do
					begin
						AddNode(n, x);
						n := n.NextSibling;
					end;
			end;
	end;

//var EventMask: integer;
begin
  //EventMask := SendMessage(tvNodes.Handle, EM_SETEVENTMASK, 0, 0);
	tvNodes.Items.BeginUpdate;
	try
		if FIncTree then
			tvNodes.Items.AddObject(nil, XMLDoc.Document.Description, XMLDoc.Document).HasChildren := true
		else
			begin
				Screen.Cursor := crHourGlass;
				try
					AddNode(XMLDoc.Document, nil);
				finally
					Screen.Cursor := crDefault;
				end;
			end;
	finally
		tvNodes.Items.EndUpdate;
    //SendMessage(tvNodes.Handle, EM_SETEVENTMASK, 0, EventMask);
	end;
end;

procedure TfmXMLTestMain.tvNodesExpanding(Sender: TObject;
	Node: TTreeNode; var AllowExpansion: Boolean);

var
	n: TXMLNode;

	procedure ExpandNode;
	var
		x: TXMLNode;
		last: TTreeNode;
	begin
		if n.ChildNodesCount > 0 then
			begin
				Screen.Cursor := crHourGlass;
				tvNodes.Items.BeginUpdate;
				LockWindowUpdate(tvNodes.Handle);
				try
					last := nil;
					x := n.FirstChild;
					while x <> nil do
						begin
							if last = nil then
								last := tvNodes.Items.AddChildObject(Node, x.Description, x)
							else
								last := tvNodes.Items.AddObject(last, x.Description, x);
							last.HasChildren := x.HasChildNodes;
							x := x.NextSibling;
						end;
				finally
					LockWindowUpdate(0);
					tvNodes.Items.EndUpdate;
					Screen.Cursor := crDefault;
				end;
			end;
	end;

begin
	if (Node <> nil) and (Node.Data <> nil) then
		begin
			n := TXMLNode(Node.Data);
			if n.HasChildNodes and (Node.Count = 0) then
				ExpandNode;
		end;
end;

procedure TfmXMLTestMain.TimeStart;
begin
	FBeginMemSize := AllocMemSize;
	QueryPerformanceCounter(FBeginTime);
end;

procedure TfmXMLTestMain.TimeStop(l: TLabel);
var
	EndMemSize: integer;
	EndTime, Freq: Int64;
begin
	EndMemSize := AllocMemSize;
	QueryPerformanceCounter(EndTime);
	QueryPerformanceFrequency(Freq);
	l.Caption := Format('%.3n ms, %.3n KB', [(EndTime - FBeginTime) * 1000 / Freq,
		(EndMemSize - FBeginMemSize) / 1024]);
	l.Update;
	Update;
end;

procedure TfmXMLTestMain.SetStatus(s: string);
begin
  sbStatus.Panels[0].Text := s;
end;

procedure TfmXMLTestMain.SetNode(Node: TXMLNode);

	procedure Add(Prop, Value: string; NewLine: boolean);
	begin
		meNode.SelAttributes.Style := [fsBold];
		if NewLine then
			meNode.SelText := Prop + ':'#13#10
		else
			meNode.SelText := Prop + ' = ';
		meNode.SelAttributes.Style := [];
		meNode.SelText := Value + #13#10;
	end;

begin
	Caption := Node.Description;
	meNode.Clear;

	Add('NodeName', Node.NodeName, false);
	Add('NodeValue', Node.NodeValue, false);
	Add('NodeType', TXMLNodeTypeStrings[Node.NodeType], false);
	if Node.ParentNode = nil then
		Add('ParentNode', '', false)
	else
		Add('ParentNode', Node.ParentNode.Description, false);
	Add('Path', Node.Path, false);

	Add('ChildNodesCount', IntToStr(Node.ChildNodesCount), false);
	Add('TotalNodesCount', IntToStr(Node.TotalNodesCount), false);
	if Node.HasChildNodes then
		Add('HasChildNodes', 'true', false)
	else
		Add('HasChildNodes', 'false', false);
	Add('Data', Node.Data, false);

	if Node.TotalNodesCount > 100 then
		begin
			meNode.Lines.Add('');
			meNode.Lines.Add('Total Nodes Count is > 100. Text, ExactText, Xml properties will not be shown here for such a large node... ');
		end
	else
		begin
			Add('Text', Node.Text, true);
			Add('ExactText', Node.ExactText, true);
			Add('Xml', Node.Xml, true);
		end;
end;

procedure TfmXMLTestMain.btnFullExpandClick(Sender: TObject);
begin
	tvNodes.FullExpand
end;

procedure TfmXMLTestMain.opdXMLFileShow(Sender: TObject);
//var	Toolbar: HWND;
begin
(*
	A001 directory one level up
	A002 create a new directory
	A003 list view (default)
	A004 report view
	A005 drop down directory combo box
*)
	//Toolbar := FindWindowEx(GetParent((Sender as TOpenDialog).Handle), 0, 'ToolbarWindow32', nil);
	//PostMessage(Toolbar, WM_COMMAND, $A004, 0);
	PostMessage(GetParent((Sender as TOpenDialog).Handle), WM_COMMAND, $A004, 0);
end;

procedure TfmXMLTestMain.sbStatusResize(Sender: TObject);
begin
	sbStatus.Panels[0].Width := sbStatus.Width - (sbStatus.Panels[1].Width + sbStatus.Panels[2].Width);
	pbProgress.Width := sbStatus.Panels[2].Width-2;
	pbProgress.Height := sbStatus.Height - 2;
	pbProgress.Top := 2;
	pbProgress.Left := sbStatus.Width - pbProgress.Width;
end;

procedure TfmXMLTestMain.XMLDocReadProgress(Sender: TObject;
	Reader: TXMLReader);
begin
	sbStatus.Panels[1].Text := IntToStr(Reader.ProgressPercent)+'%, ' +
		FloatToStrF((Reader.Position+1023) div 1024, ffNumber, 15, 0) + ' KB';
	sbStatus.Update;
	pbProgress.Position := Reader.ProgressPercent;
	pbProgress.Update;

	Application.ProcessMessages;
end;

procedure TfmXMLTestMain.XMLDocWriteProgress(Sender: TObject;
	Writer: TXMLWriter);
begin
	sbStatus.Panels[1].Text := IntToStr(Writer.ProgressPercent)+'%, ' +
		FloatToStrF((Writer.Position+1023) div 1024, ffNumber, 15, 0) + ' KB';
	sbStatus.Update;
	pbProgress.Position := Writer.ProgressPercent;
	pbProgress.Update;

	Application.ProcessMessages;
end;

procedure TfmXMLTestMain.sbNodeClick(Sender: TObject);
begin
	meNode.Visible := sbNode.Down;
	spNode.Visible := sbNode.Down;
	if (tvNodes.Selected <> nil) and (tvNodes.Selected.Data <> nil) and meNode.Visible then
		SetNode(TXMLNode(tvNodes.Selected.Data));
end;

procedure TfmXMLTestMain.tvNodesChange(Sender: TObject; Node: TTreeNode);
begin
	if (Node <> nil) and (Node.Data <> nil) and meNode.Visible then
		begin
			SetNode(TXMLNode(Node.Data));
		end;
end;

procedure TfmXMLTestMain.acOpenUpdate(Sender: TObject);
begin
	//
end;

procedure TfmXMLTestMain.acOpenExecute(Sender: TObject);
begin
	if not opdXMLFile.Execute then Exit;
	tvNodes.Items.Clear;

	// We clear the content before load so that freeing large XML object
	// inside of Load would not increase time required to complete the load
	XMLDoc.Clear;

	TimeStart;
	// Check HTML file reader options
	XMLDoc.HTMLReader := UpperCase(ExtractFileExt(opdXMLFile.FileName)) <> '.XML';
	XMLDoc.LoadFromFile(opdXMLFile.FileName);
	//XMLDoc.LoadFromString(ReadMemo(opdXMLFile.FileName));
	TimeStop(laTimeToOpen);

	SetStatus(opdXMLFile.FileName + ', open complete. TotalNodesCount = ' + IntToStr(XMLDoc.Document.TotalNodesCount)+'.');
	UpdateTreeView;
end;

procedure TfmXMLTestMain.acNewUpdate(Sender: TObject);
begin
	//
end;

procedure TfmXMLTestMain.acNewExecute(Sender: TObject);
var
	i, j: integer;
	items, item, n: TXMLNode;
begin
	tvNodes.Items.Clear;
	XMLDoc.Clear;
	TimeStart;
	items := XMLDoc.Document.CreateChild;
	items.NodeName := 'ITEMS';
	for i := 0 to 50000 do
		begin
			item := items.CreateChild;
			item.NodeName := 'ITEM';
			//item.Data := 'Data' + IntToStr(i);
			for j := 1 to 5 do
				begin
					n := item.CreateChild;
					n.NodeName := 'DATA' + IntToStr(j);
					n.NodeValue := 'Data <]><]]><]]]>' + IntToStr(i) + '-' + IntToStr(j);
				end;
		end;
	TimeStop(laTimeToOpen);

	SetStatus('New complete, TotalNodesCount = ' + IntToStr(XMLDoc.Document.TotalNodesCount)+'.');
	UpdateTreeView;
end;

procedure TfmXMLTestMain.acSaveUpdate(Sender: TObject);
begin
	TAction(Sender).Enabled := not XMLDoc.IsEmpty;
end;

procedure TfmXMLTestMain.acSaveExecute(Sender: TObject);
begin
	if not svdXMLFile.Execute then Exit;
	TimeStart;
	XMLDoc.SaveToFile(svdXMLFile.FileName);
	TimeStop(laTimeToSave);
	SetStatus(svdXMLFile.FileName + ', save complete.');
end;

procedure TfmXMLTestMain.acFreeUpdate(Sender: TObject);
begin
	TAction(Sender).Enabled := not XMLDoc.IsEmpty;
end;

procedure TfmXMLTestMain.acFreeExecute(Sender: TObject);
begin
	tvNodes.Items.Clear;
	TimeStart;
	XMLDoc.Clear;
	TimeStop(laTimeToFree);
	SetStatus('Free complete.');
end;

procedure TfmXMLTestMain.acWriteTestUpdate(Sender: TObject);
begin
	//
end;

procedure TfmXMLTestMain.acWriteTestExecute(Sender: TObject);
var
	XML: TXMLFileWriter;
begin
	if not svdXMLFile.Execute then Exit;
	// this code is just a simple demo of the TXMLWriter functionality
	XML := TXMLFileWriter.Create(svdXMLFile.FileName);
	try
		// this example creates a near copy of the sample.xml file used in Charlie
		// Calvert's XML demos which can be found at:
		// http://homepages.borland.com/ccalvert/TechPapers/Delphi/XMLBrowse/index.htm
		XML.StartDoc;
		//XML.WriteStandaloneDocumentDeclaration;
		XML.WriteTag('HTML');
		XML.WriteTag('HEAD');
		XML.WriteData('TITLE', 'Sample XML File');
		XML.WriteTag('HEAD', xttEnding);
		XML.WriteTag('BODY');
		XML.WriteData('P', 'Right under this line I insert an XML data island.');
		XML.WriteTag('XML', xttStarting, True);
		XML.WriteTagParam('ID', 'CDXML');
		XML.WriteTagClose;
		XML.WriteTag('CDS');
		XML.WriteData('CD', 'Two Against Nature');
		XML.WriteData('CD', 'Giant Steps');
		XML.WriteData('CD', 'Round About Midnight');
		XML.WriteData('CD', 'Imaginary Day');
		XML.WriteTag('CDS', xttEnding);
		XML.WriteTag('XML', xttEnding);
		XML.WriteTag('BODY', xttEnding);
		XML.WriteTag('HTML', xttEnding);
		XML.EndDoc;
	finally
		XML.Free;
	end;
end;

procedure TfmXMLTestMain.acMSXMLUpdate(Sender: TObject);
begin
	//
end;

procedure TfmXMLTestMain.acMSXMLExecute(Sender: TObject);

	function NodeText(Node: IXMLDOMNode): string;
	begin
		result := Node.nodeName + ': ' + Node.nodeTypeString;
		if Node.nodeValue <> null then
			result := result + ' = ' + Node.nodeValue;
	end;

	procedure AddNode(Node: IXMLDOMNode; TreeNode: TTreeNode);
	var
		x: TTreeNode;
		n: IXMLDOMNode;
		a: IXMLDOMNamedNodeMap;
		i: integer;
	begin
		x := tvNodes.Items.AddChild(TreeNode, NodeText(Node));
		(*
		DebugInfo(x.Text + '.'#13#10 +
			'  Text = [' + StringReplace(Node.Text, #10, #13#10, [rfReplaceAll]) + ']'#13#10 +
			'  XML = ' + Node.XML + #13#10);
		*)

		a := Node.attributes;
		if a <> nil then
			begin
				i := 0;
				repeat
					n := a.item[i];
					if n <> nil then
						AddNode(n, x);
					Inc(i);
				until n = nil;
			end;

		n := Node.firstChild;
		while n <> nil do
			begin
				AddNode(n, x);
				n := n.nextSibling;
			end;
	end;

var
	d: DOMDocument;
begin
	// MSXML parser
	if not opdXMLFile.Execute then Exit;

	tvNodes.Items.Clear;
	TimeStart;
	d := CoDOMDocument.Create;
	d.loadXML(ReadMemo(opdXMLFile.FileName));
	TimeStop(laTimeToOpen);

	if not FIncTree or
		 (MessageDlg('MSXML DOM is loaded. Incremental tree fill is not available, do you want to fill full DOM tree?',
			mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
		begin
			tvNodes.Items.BeginUpdate;
			Screen.Cursor := crHourGlass;
			try
				AddNode(d, nil);
			finally
				Screen.Cursor := crDefault;
				tvNodes.Items.EndUpdate;
			end;
		end;
end;

procedure TfmXMLTestMain.acIncTreeUpdate(Sender: TObject);
begin
	TAction(Sender).Checked := FIncTree;
end;

procedure TfmXMLTestMain.acIncTreeExecute(Sender: TObject);
begin
	FIncTree := not FIncTree;
end;

procedure TfmXMLTestMain.acPreserveWhiteSpaceUpdate(Sender: TObject);
begin
	TAction(Sender).Checked := XMLDoc.PreserveWhiteSpace;
end;

procedure TfmXMLTestMain.acPreserveWhiteSpaceExecute(Sender: TObject);
begin
	XMLDoc.PreserveWhiteSpace := not (XMLDoc.PreserveWhiteSpace);
end;

procedure TfmXMLTestMain.acMixedContentUpdate(Sender: TObject);
begin
	TAction(Sender).Checked := XMLDoc.MixedContent;
end;

procedure TfmXMLTestMain.acMixedContentExecute(Sender: TObject);
begin
	XMLDoc.MixedContent := not (XMLDoc.MixedContent);
end;

procedure TfmXMLTestMain.acFormatOutputUpdate(Sender: TObject);
begin
	TAction(Sender).Checked := XMLDoc.FormatOutput;
end;

procedure TfmXMLTestMain.acFormatOutputExecute(Sender: TObject);
begin
	XMLDoc.FormatOutput := not (XMLDoc.FormatOutput);
end;

procedure TfmXMLTestMain.btnSelectClick(Sender: TObject);
var
	Node: TXMLNode;
	query: string;
begin
	if (tvNodes.Selected <> nil) and (tvNodes.Selected.Data <> nil) then
		begin
			Node := TXMLNode(tvNodes.Selected.Data);
			if InputQuery('Please enter', 'Query string', query) then
				begin
					FSelectedNodes := Node.selectNodes(query);
					tvNodes.Invalidate;
				end;
		end;
end;

procedure TfmXMLTestMain.tvNodesCustomDrawItem(Sender: TCustomTreeView;
	Node: TTreeNode; State: TCustomDrawState; var DefaultDraw: Boolean);
var
	n: TXMLNode;
	fs: TFontStyles;
begin
	n := TXMLNode(Node.Data);
	fs := [];
	if n.NodeType = xntAttribute then
		Include(fs, fsItalic);
	if IsNodeInArray(FSelectedNodes, n) then
		Include(fs, fsBold);
	Sender.Canvas.Font.Style := fs;
end;

end.

